﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Analysis.Views;
using HeuristicLab.MainForm;

namespace HeuristicLab.Problems.DataAnalysis.Views {
  [View("ExtendedHeatMap View")]
  [Content(typeof(ExtendedHeatMap), false)]
  public partial class ExtendedHeatMapView : HeatMapView {
    public new ExtendedHeatMap Content {
      get { return (ExtendedHeatMap)base.Content; }
      set { base.Content = value; }
    }

    public ExtendedHeatMapView() {
      InitializeComponent();
      chart.ChartAreas[0].AxisX.Title = "";
      chart.ChartAreas[0].AxisY.Title = "";
    }

    protected override void OnResize(EventArgs e) {
      base.OnResize(e);
      SetMarkerSize();
    }

    private void SetMarkerSize() {
      if (chart != null && chart.ChartAreas.Count > 0 && chart.Series.Count > 0) {
        int height = (chart.Size.Height / 4) * 3;
        double max = chart.ChartAreas[0].AxisY.Maximum;
        chart.Series[0].MarkerSize = (int)Math.Floor(height / max);
      }
    }

    protected override void UpdatePoints() {
      base.UpdatePoints();
      UpdateLabels();
      SetMarkerSize();
    }

    protected virtual void UpdateLabels() {
      chart.ChartAreas[0].AxisX.CustomLabels.Clear();
      IList<string> names = Content.ColumnNames.ToList();
      for (int i = 0; i < names.Count; i++) {
        chart.ChartAreas[0].AxisX.CustomLabels.Add(i + 0.5, i + 1.5, names[i]);
        chart.ChartAreas[0].AxisY.CustomLabels.Add(i + 0.5, i + 1.5, names[i]);
      }
    }

    protected override void DeregisterContentEvents() {
      Content.ColumnNamesChanged -= new EventHandler(Content_ColumnNamesChanged);
      Content.Reset -= new EventHandler(Content_Reset);
      base.DeregisterContentEvents();
    }
    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      // only column names event is required. Row names are the same.
      Content.ColumnNamesChanged += new EventHandler(Content_ColumnNamesChanged);
      Content.Reset += new EventHandler(Content_Reset);
    }

    protected virtual void Content_ColumnNamesChanged(object sender, EventArgs e) {
      if (InvokeRequired)
        Invoke(new EventHandler(Content_ColumnNamesChanged), sender, e);
      else {
        UpdateLabels();
      }
    }

    protected void Content_Reset(object sender, EventArgs e) {
      UpdatePoints();
    }
  }
}
